@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2007 Stig Venaas <venaas@uninett.no>
- * Copyright (C) 2010 NORDUnet A/S
+ * Copyright (C) 2010,2011 NORDUnet A/S
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -28,6 +28,9 @@ static uint8_t debug_level = DBG_INFO;
 static char *debug_filepath = NULL;
 static FILE *debug_file = NULL;
 static int debug_syslogfacility = 0;
+#if defined(WANT_FTICKS)
+static int fticks_syslogfacility = 0;
+#endif
 static uint8_t debug_timestamp = 0;
 
 void debug_init(char *ident) {
@@ -64,40 +67,60 @@ uint8_t debug_get_level() {
     return debug_level;
 }
 
-int debug_set_destination(char *dest) {
-    static const char *facstrings[] = { "LOG_DAEMON", "LOG_MAIL", "LOG_USER", "LOG_LOCAL0",
-					"LOG_LOCAL1", "LOG_LOCAL2", "LOG_LOCAL3", "LOG_LOCAL4",
-					"LOG_LOCAL5", "LOG_LOCAL6", "LOG_LOCAL7", NULL };
-    static const int facvals[] = { LOG_DAEMON, LOG_MAIL, LOG_USER, LOG_LOCAL0,
-				   LOG_LOCAL1, LOG_LOCAL2, LOG_LOCAL3, LOG_LOCAL4,
-				   LOG_LOCAL5, LOG_LOCAL6, LOG_LOCAL7 };
+int debug_set_destination(char *dest, int log_type) {
+    static const char *facstrings[] = {
+        "LOG_DAEMON", "LOG_MAIL", "LOG_USER", "LOG_LOCAL0",
+	"LOG_LOCAL1", "LOG_LOCAL2", "LOG_LOCAL3", "LOG_LOCAL4",
+	"LOG_LOCAL5", "LOG_LOCAL6", "LOG_LOCAL7", NULL };
+    static const int facvals[] = {
+        LOG_DAEMON, LOG_MAIL, LOG_USER, LOG_LOCAL0,
+	LOG_LOCAL1, LOG_LOCAL2, LOG_LOCAL3, LOG_LOCAL4,
+	LOG_LOCAL5, LOG_LOCAL6, LOG_LOCAL7 };
     extern int errno;
     int i;
 
     if (!strncasecmp(dest, "file:///", 8)) {
-	debug_filepath = stringcopy(dest + 7, 0);
-	debug_file = fopen(debug_filepath, "a");
-	if (!debug_file) {
-	    debug_file = stderr;
-	    debugx(1, DBG_ERR, "Failed to open logfile %s\n%s",
-		   debug_filepath, strerror(errno));
+	if (log_type != LOG_TYPE_FTICKS) {
+	    debug_filepath = stringcopy(dest + 7, 0);
+	    debug_file = fopen(debug_filepath, "a");
+	    if (!debug_file) {
+	        debug_file = stderr;
+	        debugx(1, DBG_ERR, "Failed to open logfile %s\n%s",
+                       debug_filepath, strerror(errno));
+	    }
+	    setvbuf(debug_file, NULL, _IONBF, 0);
+	} else {
+	    debug(DBG_WARN, "FTicksSyslogFacility starting with file:/// not "
+                  "permitted, assuming default F-Ticks destination");
 	}
-	setvbuf(debug_file, NULL, _IONBF, 0);
 	return 1;
     }
-    if (!strncasecmp(dest, "x-syslog://", 11)) {
-	dest += 11;
-	if (*dest == '/')
-	    dest++;
+    if (!strncasecmp(dest, "x-syslog://", 11) || log_type == LOG_TYPE_FTICKS) {
+	if (!strncasecmp(dest, "x-syslog://", 11)) {
+            dest += 11;
+            if (*dest == '/')
+                dest++;
+	}
 	if (*dest) {
 	    for (i = 0; facstrings[i]; i++)
 		if (!strcasecmp(dest, facstrings[i]))
 		    break;
 	    if (!facstrings[i])
 		debugx(1, DBG_ERR, "Unknown syslog facility %s", dest);
-	    debug_syslogfacility = facvals[i];
-	} else
-	    debug_syslogfacility = LOG_DAEMON;
+	    if (log_type != LOG_TYPE_FTICKS)
+		debug_syslogfacility = facvals[i];
+#if defined(WANT_FTICKS)
+            else if (log_type == LOG_TYPE_FTICKS)
+		fticks_syslogfacility = facvals[i];
+#endif
+	} else {
+            if (log_type != LOG_TYPE_FTICKS)
+                debug_syslogfacility = LOG_DAEMON;
+#if defined(WANT_FTICKS)
+            else if (log_type == LOG_TYPE_FTICKS)
+                fticks_syslogfacility = 0;
+#endif
+    	}
 	openlog(debug_ident, LOG_PID, debug_syslogfacility);
 	return 1;
     }
@@ -213,6 +236,20 @@ void debugerrnox(int err, uint8_t level, char *format, ...) {
     exit(err);
 }
 
+#if defined(WANT_FTICKS)
+void fticks_debug(const char *format, ...) {
+    int priority;
+    va_list ap;
+    va_start(ap, format);
+    if (!debug_syslogfacility && !fticks_syslogfacility)
+    	debug_logit(0xff, format, ap);
+    else {
+    	priority = LOG_DEBUG | fticks_syslogfacility;
+    	vsyslog(priority, format, ap);
+    	va_end(ap);
+    }
+}
+#endif
 /* Local Variables: */
 /* c-file-style: "stroustrup" */
 /* End: */
